<?php

/**
 * Title         : Aqua Resizer (Asura Resizer)
 * Description   : Resizes WordPress images on the fly
 * Version       : 1.2.0 (改)
 * Author        : Syamil MJ
 * Author URI    : http://aquagraphite.com
 * License       : WTFPL - http://sam.zoy.org/wtfpl/
 * Documentation : https://github.com/sy4mil/Aqua-Resizer/
 *
 * @param string  $url      - (required) must be uploaded using wp media uploader
 * @param int     $width    - (required)
 * @param int     $height   - (optional)
 * @param bool    $crop     - (optional) default to soft crop
 * @param bool    $single   - (optional) returns an array if false
 * @param bool    $upscale  - (optional) resizes smaller images
 * @uses  wp_upload_dir()
 * @uses  image_resize_dimensions()
 * @uses  wp_get_image_editor()
 *
 * @return str|array
 */

if(!class_exists('Aq_Resize')) {
    class Aq_Resize
    {
        /**
         * The singleton instance
         */
        static private $instance = null;

        /**
         * No initialization allowed
         */
        private function __construct() {}

        /**
         * No cloning allowed
         */
        private function __clone() {}

        /**
         * For your custom default usage you may want to initialize an Aq_Resize object by yourself and then have own defaults
         */
        static public function getInstance() {
            if(self::$instance == null) {
                self::$instance = new self;
            }

            return self::$instance;
        }

        /**
         * Run, forest.
         */
        public function process( $url, $width = null, $height = null, $copy = true , $crop = null, $single = true, $upscale = false )
        {
            // Validate inputs.
            if ( ! $url || ( ! $width && ! $height ) ) return false;

            // Caipt'n, ready to hook.
            if ( true === $upscale ) add_filter( 'image_resize_dimensions', array($this, 'aq_upscale'), 10, 6 );

            // Define upload path & dir.
            $upload_info = wp_upload_dir();
            $upload_dir = $upload_info['basedir'];
            $upload_url = $upload_info['baseurl'];
            
            $http_prefix = "http://";
            $https_prefix = "https://";
            
            /* if the $url scheme differs from $upload_url scheme, make them match 
               if the schemes differe, images don't show up. */
            if(!strncmp($url,$https_prefix,strlen($https_prefix))){ //if url begins with https:// make $upload_url begin with https:// as well
                $upload_url = str_replace($http_prefix,$https_prefix,$upload_url);
            }
            elseif(!strncmp($url,$http_prefix,strlen($http_prefix))){ //if url begins with http:// make $upload_url begin with http:// as well
                $upload_url = str_replace($https_prefix,$http_prefix,$upload_url);      
            }

            // Check if $img_url is local.
            if ( false === strpos( $url, $upload_url ) ) return false;

            // Define path of image.
            $rel_path = str_replace( $upload_url, '', $url );
            $img_path = $upload_dir . $rel_path;


            //ローカルホストであればパスを変換
            if( preg_match( '/^[a-zA-Z]:\\\/' , $img_path ) )
            {
                $img_path = str_replace( '\\' , '/' , $img_path );
            }

            // Check if img path exists, and is an image indeed.
            if ( ! file_exists( $img_path ) or ! getimagesize( $img_path ) ) return false;

            // Get image info.
            $info                       = pathinfo( $img_path );
            $ext                        = $info[ 'extension' ];
            list( $orig_w , $orig_h )   = getimagesize( $img_path );

            // Get image size after cropping.
            $dims                       = image_resize_dimensions( $orig_w, $orig_h, $width, $height, $crop );
            $dst_w                      = $dims[4];
            $dst_h                      = $dims[5];


            //既にリサイズする必要がなければ
            if( ! $dims && ( ( ( null === $height && $orig_w == $width ) xor ( null === $width && $orig_h == $height ) ) xor ( $height == $orig_h && $width == $orig_w ) ) )
            {
                $img_url    = $url;
                $dst_w      = $orig_w;
                $dst_h      = $orig_h;
            }
            else
            {
                // Use this to check if cropped image already exists, so we can return that instead.


                //コピーを作るなら
                if( $copy )
                {
                    $suffix         = "{$dst_w}x{$dst_h}";

                    $dst_rel_path   = str_replace( '.' . $ext , '' , $rel_path );

                    //ファイル名フォーマット
                    $destfilename   = "{$upload_dir}{$dst_rel_path}-{$suffix}.{$ext}";
                }
                //作らないなら
                else
                {
                    $dst_rel_path   = str_replace( '.' . $ext , '' , $rel_path );

                    //ファイル名フォーマット
                    $destfilename   = "{$upload_dir}{$dst_rel_path}.{$ext}";
                }


                //リサイズできなければ終了
                if(! $dims || ( true == $crop && false == $upscale && ( $dst_w < $width || $dst_h < $height ) ) )
                {
                    // Can't resize, so return false saying that the action to do could not be processed as planned.
                    return false;
                }
                //ファイルが存在すれば
                elseif( file_exists( $destfilename ) )
                {
                    //ファイル情報を取得する
                    list( $width_ , $height_ , $type_ , $attr_ )    = getimagesize( $destfilename );

                    //既にリサイズ済みであれば
                    if( ( $width and $width == $width_ ) or ( $height and $height == $height_ ) )
                    {
                        //コピーを作るなら
                        if( $copy )
                        {
                            $img_url = "{$upload_url}{$dst_rel_path}-{$suffix}.{$ext}";
                        }
                        //作らないなら
                        else
                        {
                            $img_url = "{$upload_url}{$dst_rel_path}.{$ext}";
                        }
                    }
                    //まだリサイズされていなければ
                    else
                    {
                        //画像を保存する
                        $resized_file           = $this->SAVE_IMG( $img_path , $width , $height , $copy , $crop );

                        //エラーがあれば
                        if(! is_wp_error( $resized_file ) )
                        {
                            $resized_rel_path   = str_replace( $upload_dir , '' , $resized_file[ 'path' ] );
                            $img_url            = $upload_url . $resized_rel_path;
                        }
                        else
                        {
                            return false;
                        }

                    }
                }
                //ファイルがなければ終了
                else
                {
                    return false;
                }

            }


            // Okay, leave the ship.
            if( true === $upscale ) remove_filter( 'image_resize_dimensions' , array( $this , 'aq_upscale' ) );

            // Return the output.
            if( $single )
            {
                // str return.
                $image  = $img_url;
            }
            else
            {
                // array return.
                $image  = array( 0 => $img_url , 1 => $dst_w , 2 => $dst_h );
            }

            return $image;
        }
        /**
         * Callback to overwrite WP computing of thumbnail measures
         */
        function aq_upscale( $default, $orig_w, $orig_h, $dest_w, $dest_h, $crop )
        {
            if ( ! $crop ) return null; // Let the wordpress default function handle this.

            // Here is the point we allow to use larger image size than the original one.
            $aspect_ratio = $orig_w / $orig_h;
            $new_w = $dest_w;
            $new_h = $dest_h;

            if ( ! $new_w ) {
                $new_w = intval( $new_h * $aspect_ratio );
            }

            if ( ! $new_h ) {
                $new_h = intval( $new_w / $aspect_ratio );
            }

            $size_ratio = max( $new_w / $orig_w, $new_h / $orig_h );

            $crop_w = round( $new_w / $size_ratio );
            $crop_h = round( $new_h / $size_ratio );

            $s_x = floor( ( $orig_w - $crop_w ) / 2 );
            $s_y = floor( ( $orig_h - $crop_h ) / 2 );

            return array( 0, 0, (int) $s_x, (int) $s_y, (int) $new_w, (int) $new_h, (int) $crop_w, (int) $crop_h );
        }
        //画像を保存する///////////////////////////////////////////////////////
        function SAVE_IMG( $img_path , $width , $height , $copy , $crop )
        {

            //https://wpdocs.osdn.jp/%E9%96%A2%E6%95%B0%E3%83%AA%E3%83%95%E3%82%A1%E3%83%AC%E3%83%B3%E3%82%B9/wp_get_image_editor
            $editor             = wp_get_image_editor( $img_path );

             if( is_wp_error( $editor ) || is_wp_error( $editor->resize( $width , $height , $crop ) ) ) return false;

echo "<pre>";
var_dump("まだリサイズされてない2");
echo "</pre>";


            //コピーを作るなら
            if( $copy )
            {
echo "<pre>";
var_dump("コピーを作る2");
echo "</pre>";

                //画像を保存 コピーが作られる 元画像名-800x600.jpg
                $resized_file   = $editor->save();
            }
            //作らないなら
            else
            {
echo "<pre>";
var_dump("コピーを作らない2");
echo "</pre>";
                //画像を保存
                $resized_file   = $editor->save( $img_path );
            }
        }
    }
}





if(!function_exists('aq_resize')) {

    /**
     * This is just a tiny wrapper function for the class above so that there is no
     * need to change any code in your own WP themes. Usage is still the same :)
     */
    function aq_resize( $url, $width = null, $height = null, $copy = true , $crop = null, $single = true, $upscale = false ) {
        $aq_resize = Aq_Resize::getInstance();
        return $aq_resize->process( $url, $width, $height, $copy , $crop, $single, $upscale );
    }
}

?>